"""
Creates the application and sets it up.
"""
import os
import logging
from time import sleep
from dotenv import load_dotenv
from flask import Flask
from flask_cors import CORS
from pony.orm import Database, db_session
from pony.orm.dbapiprovider import OperationalError

# load environment variables, add interal
load_dotenv()

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# create applocation and apply variables
app = Flask(__name__)
app.config["DATA_DIR"] = "/storage/app/data"
app.config["ASSETS_DIR"] = "/storage/app/assets"
app.config.update(SECRET_KEY=os.environ["FLASK_SECRET_KEY"])
app.dotenv = os.environ
app.config["PONY"] = {
    "provider": "postgres",
    "user": os.environ["PG_USER"],
    "password": os.environ["PG_PASSWORD"],
    "host": os.environ["PG_HOST"],
    "port": os.environ["PG_PORT"],
    "database": os.environ["PG_DB"],
}


# database setup - fill with data
def setup_cities(city=None):
    try:
        if city == "bilbao":
            setup_bilbao()
        if city == "amsterdam":
            setup_amsterdam()
        if city == "helsinki":
            setup_helsinki()
        if city == "messina":
            setup_messina()
    except Exception as exception:
        logger.warning("database exists, not a problem")
        logger.warning(f"error: {exception}")


def setup_amsterdam():
    """
    Sets up the amsterdam use case database.
    """
    from app.entities import Network, Simulation, City, Status, Plan

    with db_session:
        amsterdam: City = City(
            city_id="amsterdam", display_name="Amsterdam", lat=52.3676, lon=4.9041
        )

        network: Network = Network(
            name="Amsterdam network",
            description="Amsterdam network.",
            city=amsterdam,
            done_osm_processing=True,
            donematsim_processing=True,
        )
        plan: Plan = Plan(
            name="amsterdam plan default",
            network=network,
        )
        simulation: Simulation = Simulation(
            city=amsterdam,
            name="Amsterdam baseline simulation",
            description="....",
            plan=plan,
            status=Status.CREATED,
        )

        plan_2: Plan = Plan(
            name="amsterdam plan noord",
            network=network,
        )
        simulation_2: Simulation = Simulation(
            city=amsterdam,
            name="Amsterdam Noord simulation",
            description="....",
            plan=plan_2,
            status=Status.CREATED,
        )

        app.logger.info("Amsterdam baseline")
        app.logger.info(f"Amsterdam simulation: {simulation}")
        app.logger.info(f"Amsterdam network: {network}")
        app.logger.info(f"Amsterdam plans: {plan}")

        app.logger.info("Amsterdam Noord")
        app.logger.info(f"Amsterdam Noord simulation: {simulation_2}")
        app.logger.info(f"Amsterdam Noord network: {network}")
        app.logger.info(f"Amsterdam Noord plans: {plan_2}")


def setup_bilbao():
    """
    Sets up the bilbao use case database.
    """
    from app.entities import Network, Simulation, City, Status, Plan
    import json
    from pathlib import Path

    with db_session:
        bilbao: City = City(
            city_id="bilbao",
            display_name="Bilbao",
            lat=43.262985,
            lon=-2.935013,
        )
        network: Network = Network(
            name="Bilbao network",
            description="Current Bilbao road network",
            city=bilbao,
            done_osm_processing=True,
            donematsim_processing=True,
        )
        plan: Plan = Plan(name="bilbao plan base", network=network)
        simulation: Simulation = Simulation(
            city=bilbao,
            name="Bilbao baseline simulation",
            description="Current state of Bilbao.",
            plan=plan,
            status=Status.CREATED,
        )

        network_2: Network = Network(
            name="Bilbao Moyua LTZ network",
            description="Bilbao with Moyua LTZ.",
            city=bilbao,
            done_osm_processing=True,
            donematsim_processing=True,
        )
        plan_2: Plan = Plan(name="bilbao plan ltz", network=network_2)
        simulation_2: Simulation = Simulation(
            city=bilbao,
            name="Bilbao Moyua LTZ",
            description="Bilbao with Moyua LTZ.",
            plan=plan_2,
            status=Status.CREATED,
        )
        # Find and order all simulations dir
        simulations_path = Path(
            f"{app.dotenv['DATA_DIR']}/{app.dotenv['URBANITE_CITY']}/simulations"
        )
        if os.path.exists(simulations_path):
            dirs = [x for x in simulations_path.iterdir() if x.is_dir()]
            dirs.sort(key=lambda t: int(t.name))
            app.logger.info(f"Found {dirs}")

            # Inser the simulations based on the info.json file
            for d in dirs:
                info_json_path = d / "info.json"

                if not info_json_path.exists():
                    app.logger.warning(
                        f"info.json does not exists for {d.name}! It won't be added to the DB!"
                    )
                    continue

                with open(info_json_path) as info_file:
                    info = json.load(info_file)

                app.logger.info(f"Adding: {info['name']}")

                network: Network = Network(
                    name=info["name"],
                    description=info["description"],
                    city=bilbao,
                    done_osm_processing=True,
                    donematsim_processing=True,
                )
                plan: Plan = Plan(name=info["name"], network=network)

                Simulation(
                    city=bilbao,
                    name=info["name"],
                    description=info["description"],
                    plan=plan,
                    status=Status.CREATED,
                )

                app.logger.info(f"Bilbao simulation: {info['name']}")
                app.logger.info(f"Bilbao network: {network}")
                app.logger.info(f"Bilbao plans: {plan}")
        else:
            app.logger.info(f"No simulations folder present, skipping adding simulations from info.json")


def setup_helsinki():
    """
    Sets up the helsinki use case database.
    """
    from app.entities import Network, Simulation, City, Status, Plan

    with db_session:
        helsinki: City = City(
            city_id="helsinki",
            display_name="Helsinki",
            lat=60.1699,
            lon=24.9384,
        )

        network: Network = Network(
            name="Helsinki network",
            description="Helsinki network.",
            city=helsinki,
            done_osm_processing=True,
            donematsim_processing=True,
        )
        plan: Plan = Plan(
            name="helsinki plan default",
            network=network,
        )
        simulation: Simulation = Simulation(
            city=helsinki,
            name="Helsinki baseline simulation",
            description="....",
            plan=plan,
            status=Status.CREATED,
        )

        network_2: Network = Network(
            name="Helsinki tunnel network",
            description="Helsinki network with harbor tunnel.",
            city=helsinki,
            done_osm_processing=True,
            donematsim_processing=True,
        )
        plan_2: Plan = Plan(
            name="helsinki plan tunnel",
            network=network_2,
        )
        simulation_2: Simulation = Simulation(
            city=helsinki,
            name="Helsinki harbor tunnel simulation",
            description="....",
            plan=plan_2,
            status=Status.CREATED,
        )

        app.logger.info("Helsinki baseline")
        app.logger.info(f"Helsinki simulation: {simulation}")
        app.logger.info(f"Helsinki network: {network}")
        app.logger.info(f"Helsinki plans: {plan}")

        app.logger.info("Helsinki tunnel")
        app.logger.info(f"Helsinki tunnel simulation: {simulation_2}")
        app.logger.info(f"Helsinki tunnel network: {network_2}")
        app.logger.info(f"Helsinki tunnel plans: {plan_2}")


def setup_messina():
    """
    Sets up the messina use case database.
    """
    from app.entities import Network, Simulation, City, Status, Plan

    with db_session:
        messina: City = City(
            city_id="messina", display_name="Messina", lat=38.1937, lon=15.5542
        )

        network: Network = Network(
            name="Messina network",
            description="Messina network.",
            city=messina,
            done_osm_processing=True,
            donematsim_processing=True,
        )
        plan: Plan = Plan(name="messina plan default", network=network)
        simulation: Simulation = Simulation(
            name="Messina baseline simulation",
            description="....",
            city=messina,
            plan=plan,
            status=Status.CREATED,
        )

        network_2: Network = Network(
            name="Messina PT network",
            description="Messina network with extended PT lines.",
            city=messina,
            done_osm_processing=True,
            donematsim_processing=True,
        )
        plan_2: Plan = Plan(name="messina plan PT", network=network_2)
        simulation_2: Simulation = Simulation(
            name="Messina PT simulation",
            description="....",
            city=messina,
            plan=plan_2,
            status=Status.CREATED,
        )

        app.logger.info("Messina baseline")
        app.logger.info(f"Messina simulation: {simulation}")
        app.logger.info(f"Messina network: {network}")
        app.logger.info(f"Messina plans: {plan}")

        app.logger.info("Messina PT extension")
        app.logger.info(f"Messina PT simulation: {simulation_2}")
        app.logger.info(f"Messina PT network: {network_2}")
        app.logger.info(f"Messina PT plans: {plan_2}")


def fill_db_initial(create_tables=True):
    """
    Creates database mapping and fills up the database.
    """
    from app.entities import Network, Simulation, City, Status, Plan

    if create_tables:
        db.generate_mapping(check_tables=True, create_tables=True)
    city = os.environ["URBANITE_CITY"]

    setup_cities(city=city)


# connect to database
with app.app_context():
    db = Database()

    connected = False
    count = 0
    while not connected:
        try:
            db.bind(**app.config["PONY"])
            connected = True
        except OperationalError as er:
            if count > 10:
                logger.error(f"error connecting to database 10 times\n{er}")
                quit(-1)
            count += 1
            sleep(5)

    from app.api import api_bp

    app.register_blueprint(api_bp)

    fill_db_initial()

# allow cross-origin access
CORS(app)
