"""
Implements the API for retrieving DSS results.
"""

import json

from flask import Blueprint, request, make_response, send_file
from flask import current_app as app
from pony.orm import db_session

from app import utils
from app.entities import City, Simulation, Network, Plan

api_bp = Blueprint("api", __name__)
data_dir = f"{app.dotenv['DATA_DIR']}/{app.dotenv['URBANITE_CITY']}"
assets_dir = f"{app.dotenv['ASSETS_DIR']}"
city = app.dotenv["URBANITE_CITY"]

# Endpoints for EDA


@app.route("/storage/eda/<simulation_id>/kpis", methods=["GET"])
def return_kpis(simulation_id):
    with open(
        f"{data_dir}/{simulation_id}/dss/{simulation_id}_kpis.json", mode="rb"
    ) as file:
        json_obj = json.load(file)
        return json_obj


# Results of DSS exposed


@app.route("/storage/dss/evaluated", methods=["POST"])
@db_session
def evaluated_kpis_many():
    """
    Exposes API for retrieving DEXi evaluated KPIs for simulations.
    :body: { ids: [1, ]}
    :returns: { 1: { kpis }, }
    """
    sim_ids = json.loads(request.data.decode("utf-8"))["ids"]

    app.logger.debug("Request data: " + str(sim_ids))
    sim_dir_names = utils.get_sim_dir_names(sim_ids)
    jsons = {}
    for sim_id, sim_dir_name in zip(sim_ids, sim_dir_names):
        with open(
            f"{data_dir}/simulations/{sim_id}/results/{sim_dir_name}/evaluated.json",
            "r",
        ) as file:
            json_obj = json.loads(file.read())
            jsons[sim_id] = json_obj

    meta_path = f"{assets_dir}/kpi_hierarchy.json"
    app.logger.debug("meta path %s", meta_path)
    with open(meta_path) as f:
        metadata = json.load(f)
    return {"data": jsons, "metadata": metadata[city]}


@app.route("/storage/dss/kpis", methods=["POST"])
@db_session
def kpis_many():
    """
    Exposes API for retrieving calculated KPIs for simulations.
    :body: { ids: [1, ]}
    :returns: { 1: { evaluated kpis }, }
    """
    jsons = {}
    sim_ids = request.data
    sim_dir_names = utils.get_sim_dir_names(sim_ids)
    for sim_id, sim_dir_name in zip(sim_ids, sim_dir_names):
        with open(f"{data_dir}/{sim_id}/{sim_dir_name}/dss/kpis.json", "r") as file:
            json_obj = json.loads(file.read())
            jsons[sim_id] = json_obj
    return jsons


@app.route("/storage/populations/<net_id>")
@db_session
def get_populations(net_id):
    net = Network.select(lambda net: net.id == net_id).first()
    plans = Plan.select(lambda plan: plan.network == net)
    retval = []
    for p in plans:
        path = f"{data_dir}/original_input/{net.id}/plans/{p.id}/plans.xml"
        retval.append((path, p))
    return json.dumps(retval)


@app.route("/storage/plans/<int:network_id>")
@db_session
def get_network_id_simulations(network_id):
    retval = {}
    net = Network.select(lambda net: net.id == network_id)
    plans = []
    for n in net:
        plans += Plan.select(lambda plan: plan.network == n)
    for plan in plans:
        plan_json = {}
        plan_json["plan_name"] = plan.name
        plan_json["network"] = plan.network.name
        plan_json["simulation"] = plan.simulation.name
        retval[plan.id] = plan_json
    return json.dumps(retval)


# checks if writing to common data access works


@app.route("/storage/test/make_file")
def test_make_file():
    """
    Creates a test file.
    Storage can read from it.
    Available on `${storage.url}:${storage.port}/test/read_file`
    """
    with open(f"{data_dir}/test.txt", "w") as test_fd:
        data = map(lambda item: item + "\n", ["test", "new line", "last line"])
        test_fd.writelines(
            data,
        )
        return {"success": True}
    return {"success": False}


@app.route("/storage/test/change_file")
def test_change_file():
    """
    Adds content to a test file.
    Storage component can read from it.
    Available on `${storage.url}:${storage.port}/test/read_file`
    """
    with open(f"{data_dir}/test.txt", "a") as test_fd:
        data = map(
            lambda item: item + "\n",
            ["another line", "another new line", "last new line"],
        )
        test_fd.writelines(
            data,
        )
        return {"success": True}
    return {"success": False}


@app.route("/storage/test/show_db")
def test():
    retval = {"cities": [], "simulations": [], "networks": [], "plans": []}
    with db_session:
        for city in City.select():
            retval["cities"].append(city.city_id)
        for simulation in Simulation.select():
            retval["simulations"].append([simulation.id, simulation.name])
        for network in Network.select():
            retval["networks"].append([network.id, network.name])
        for plan in Plan.select():
            retval["plans"].append([plan.id, plan.name])
    return json.dumps(retval)


@app.route("/storage/ml-experiments")
def download_ml_experiments_file():
    root_dir = f"{app.dotenv['DATA_DIR']}"
    try:
        res = make_response(
            send_file(
                f"{root_dir}/ML experiments.pptx",
                as_attachment=True,
                attachment_filename="ML experiments.pptx",
            )
        )
        return res
    except FileNotFoundError:
        return {"error": "PPTX File not found"}


@app.route("/storage/network/<int:network_id>")
def get_network(network_id):
    return send_file(f"{data_dir}/original_input/networks/{network_id}/network.xml")


@app.route("/storage/results/<int:sim_id>")
def get_results(sim_id):
    date = utils.get_sim_dir_names([sim_id])[0]
    events_path = (
        f"{data_dir}/simulations/" + f"{sim_id}/results/{date}/output_events.xml"
    )
    app.logger.warn(f"path: {events_path}")
    return send_file(events_path)
